use strict;
use Time::HiRes                                       qw( time );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::File qw ( GetFileContent );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::GoogleBase::API::CSV::GoogleBaseProductsExportDriver;
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByPath
    JoinObjectPath
    LoadObjectByPath
    LoadRootObject
);
use DE_EPAGES::Core::API::Warning qw ( BlockWarnings );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use File::Temp qw ( tempdir tempfile );
use Readonly;
use Test::More;

Readonly my $ROOTPATH => '/TestShop';

sub Main {## no critic (ProhibitFunctionSize)
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            # startUp
            ok( Init(), 'initialization' );
            my $StartTime;
            my $FinishingTime;
            # tests
            eval {
                ok (my $PortalConfig = LoadObjectByPath($ROOTPATH)->path( JoinObjectPath( [ 'TestShop', 'PortalConfigs', 'GoogleBase_de_DE' ] ) ), 'Load portal config');
                isa_ok( $PortalConfig, 'DE_EPAGES::ProductPortal::API::Object::PortalConfigWithUpload' );
                my $dir = tempdir( CLEANUP => 1 );
                my (undef, $FileName) = tempfile( DIR => $dir );
                $StartTime = time;
                my $aWarnigns = BlockWarnings( sub {
                    $PortalConfig->exportPortal($FileName, 1);
                });

                $FinishingTime = time;
                note (sprintf('The Export needed %.4f seconds ', $FinishingTime - $StartTime));
                ok (-f $FileName,'export file created');
                my $FileContent = ${GetFileContent($FileName)};
                my $aWarningMessages = join("\n", map {$_->message} @{$aWarnigns});

                #- Kaffee 1000g ------------------------------------------------------------------
                compareRegexToFileContent($FileContent, 'Kaffee 1000g_de_DE\s*Eureka', 1,
                            'Kaffee 1000g Found' );

                #- Kaffee 500g ------------------------------------------------------------------
                compareRegexToFileContent($FileContent, 'Kaffee 500g_de_DE\s*', 0,
                            'Kaffee 500g is not exported due to an error');

                #- Gartenstuhl 1 1 1 --------------------------------------------------------------
                compareRegexToFileContent($FileContent, 'Gartenstuhl 1 1 1_de_DE\s*Eureka', 0,
                            'Gartenstuhl 1 1 1 is not exported due to an error');

                #- Gartenstuhl 3 3 1   --------------------------------------------------------------
                compareRegexToFileContent($FileContent, 'Gartenstuhl 3 3 1_de_DE\s*', 1,
                            'Gartenstuhl 3 3 1 found');

                #- Gartenstuhl 3 3 3  --------------------------------------------------------------
                compareRegexToFileContent($FileContent, 'Gartenstuhl 3 3 3_de_DE\s*Eureka', 0,
                            'Gartenstuhl 3 3 3 is not exported due to an error');

                #- Test Identifier Exist
                compareRegexToFileContent($FileContent, '3001234567892.*FALSE', 1,
                    'Product with identifier_exists set to FALSE');

                #- Test unit_pricing_measure and unit_pricing_base_measure
                compareRegexToFileContent($aWarningMessages , 'has reference unit misconfigurated', 1,
                    'Product have reference unit misconfigurated.');

                compareRegexToFileContent($aWarningMessages , 'has reference amount misconfigurated', 2,
                    'Product have amount unit misconfigurated.');

                compareRegexToFileContent($FileContent, 'Kaffee 2kg_de_DE.*2kg\s1kg', 1,
                    'unit_pricing_measure and unit_pricing_base_measure is set correctly.');
             };
             if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'Error: ' . $Error->longMessage );
            }
            # shutDown
            ok( Cleanup(), 'cleanup' );
            done_testing();
        }
    );
}

sub compareRegexToFileContent {
    my ($FileContent, $regEx, $numberResults, $comment) = @_;
    my @RegExResult = ($FileContent =~ m/$regEx/g);
    is (scalar @RegExResult,$numberResults,$comment);
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => 'TestShop' } );
    ImportObjects( 'import.xml', $ROOTPATH );
    return 1;
}

RunScript( Debug => 1, Sub => \&Main );

1;